package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.exception.ExpiredSalesDeadlineException;
import br.com.rodrigosieja.brasilprev.exception.InvalidAgeException;
import br.com.rodrigosieja.brasilprev.exception.InvalidInitialIntakeException;
import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ClienteService;
import br.com.rodrigosieja.brasilprev.service.PlanoService;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;
import java.time.Period;
import java.util.List;

@RestController
@RequestMapping("/plano")
public class PlanoController {

    private final PlanoService planoService;
    private final ProdutoService produtoService;
    private final ClienteService clienteService;

    @Autowired
    public PlanoController(PlanoService planoService,
                           ProdutoService produtoService,
                           ClienteService clienteService) {
        this.produtoService = produtoService;
        this.planoService = planoService;
        this.clienteService = clienteService;
    }

    @PostMapping
    @Operation(summary = "Contratação de Plano")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description = "Plano contratado"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<GenericResponseDTO> contratacao(final @RequestBody Plano plano) {
        Produto produto = produtoService.busca(plano.getIdProduto());
        Cliente cliente = clienteService.busca(plano.getIdCliente());

        boolean prazoVendaExpirado = planoService.prazoVendaExpirado(produto.getExpiracaoDeVenda());
        if (prazoVendaExpirado) {
            throw new ExpiredSalesDeadlineException("Prazo de venda expirado!");
        }

        int clienteIdade = Period.between(cliente.getDataDeNascimento(), LocalDate.now()).getYears();
        boolean idadeValida = planoService.idadeValida(
                clienteIdade,
                produto.getIdadeDeEntrada(),
                produto.getIdadeDeSaida()
        );
        if (idadeValida) {
            boolean aporteInicialValido = planoService.aporteInicialValido(
                    plano.getAporte(),
                    produto.getValorMinimoAporteInicial()
            );

            if (aporteInicialValido) {
                GenericResponseDTO genericResponse = new GenericResponseDTO(planoService.insere(plano).getId());
                return ResponseEntity.status(201).body(genericResponse);
            } else {
                throw new InvalidInitialIntakeException("Aporte inicial inválido para a contratação do plano!");
            }
        } else {
            throw new InvalidAgeException("Idade inválida para a contratação do plano!");
        }
    }

    @GetMapping("/buscar")
    @Operation(summary = "Lista de planos")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Planos encontrados"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<List<Plano>> lista() {
        return ResponseEntity.ok().body(planoService.lista());
    }

}
