package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.enums.Genero;
import br.com.rodrigosieja.brasilprev.exception.ExpiredSalesDeadlineException;
import br.com.rodrigosieja.brasilprev.exception.InvalidAgeException;
import br.com.rodrigosieja.brasilprev.exception.InvalidInitialIntakeException;
import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ClienteService;
import br.com.rodrigosieja.brasilprev.service.PlanoService;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
public class PlanoControllerTest {

    @Mock
    private PlanoService planoService;

    @Mock
    private ProdutoService produtoService;

    @Mock
    private ClienteService clienteService;

    @InjectMocks
    private PlanoController planoController;

    @Mock
    private Plano planoRequest;

    @Mock
    private Plano planoResponse;

    @Mock
    private Produto produtoResponse;

    @Mock
    private Cliente clienteResponse;

    private UUID idPlano;
    private UUID idCliente;
    private UUID idProduto;

    @BeforeEach
    void setup() {
        idCliente = UUID.randomUUID();
        idProduto = UUID.randomUUID();

        planoRequest = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(1000.0),
                LocalDate.of(2024, 12, 4),
                60
        );

        idPlano = UUID.randomUUID();

        planoResponse = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(1000.0),
                LocalDate.of(2024, 12, 4),
                60
        );

        planoResponse.setId(idPlano);
    }

    @Test
    public void contratacaoTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2024, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );
        clienteResponse.setId(idCliente);

        when(planoService.insere(any())).thenReturn(planoResponse);
        when(produtoService.busca(any())).thenReturn(produtoResponse);
        when(clienteService.busca(any())).thenReturn(clienteResponse);

        when(planoService.prazoVendaExpirado(any())).thenReturn(false);
        when(planoService.idadeValida(any(), any(), any())).thenReturn(true);
        when(planoService.aporteInicialValido(any(), any())).thenReturn(true);

        ResponseEntity<GenericResponseDTO> response = planoController.contratacao(planoRequest);
        assertThat(response.getStatusCode().value()).isEqualTo(201);
    }

    @Test
    public void contratacaoPrazoVendaExpiradoTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );
        clienteResponse.setId(idCliente);

        when(produtoService.busca(any())).thenReturn(produtoResponse);
        when(clienteService.busca(any())).thenReturn(clienteResponse);

        when(planoService.prazoVendaExpirado(any())).thenReturn(true);

        assertThrows(
                ExpiredSalesDeadlineException.class,
                () -> planoController.contratacao(planoRequest),
                "Expected contratacao(planoRequest) to throw ExpiredSalesDeadlineException, but it didn't"
        );
    }

    @Test
    public void contratacaoAporteInicialInvalidoTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );
        clienteResponse.setId(idCliente);

        when(produtoService.busca(any())).thenReturn(produtoResponse);
        when(clienteService.busca(any())).thenReturn(clienteResponse);

        when(planoService.prazoVendaExpirado(any())).thenReturn(false);
        when(planoService.idadeValida(any(), any(), any())).thenReturn(true);
        when(planoService.aporteInicialValido(any(), any())).thenReturn(false);

        assertThrows(
                InvalidInitialIntakeException.class,
                () -> planoController.contratacao(planoRequest),
                "Expected contratacao(planoRequest) to throw InvalidInitialIntakeException, but it didn't"
        );
    }

    @Test
    public void contratacaoIdadeInvalidaTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );
        clienteResponse.setId(idCliente);

        when(produtoService.busca(any())).thenReturn(produtoResponse);
        when(clienteService.busca(any())).thenReturn(clienteResponse);

        when(planoService.prazoVendaExpirado(any())).thenReturn(false);
        when(planoService.idadeValida(any(), any(), any())).thenReturn(false);

        assertThrows(
                InvalidAgeException.class,
                () -> planoController.contratacao(planoRequest),
                "Expected contratacao(planoRequest) to throw InvalidAgeException, but it didn't"
        );
    }

    @Test
    public void listaTest() {
        ResponseEntity<List<Plano>> response = planoController.lista();
        assertThat(response.getStatusCode().value()).isEqualTo(200);
    }

}
