package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.exception.CancelledPlanException;
import br.com.rodrigosieja.brasilprev.exception.InvalidRedemptionException;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.model.dto.ResgateDTO;
import br.com.rodrigosieja.brasilprev.service.PlanoService;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import br.com.rodrigosieja.brasilprev.service.ResgateService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
public class ResgateControllerTest {

    @Mock
    private ResgateService resgateService;

    @Mock
    private PlanoService planoService;

    @Mock
    private ProdutoService produtoService;

    @InjectMocks
    private ResgateController resgateController;

    @Mock
    private ResgateDTO resgateRequest;

    @Mock
    private Plano planoResponse;

    @Mock
    private Plano planoResgateResponse;

    @Mock
    private Produto produtoResponse;

    private UUID idPlano;
    private UUID idCliente;
    private UUID idProduto;

    @BeforeEach
    void setup() {
        idPlano = UUID.randomUUID();
        idCliente = UUID.randomUUID();
        idProduto = UUID.randomUUID();

        resgateRequest = new ResgateDTO(
                idPlano,
                BigDecimal.valueOf(100.0)
        );

        planoResponse = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(1000.0),
                LocalDate.of(2024, 12, 4),
                60
        );

        planoResgateResponse = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(900.0),
                LocalDate.of(2024, 12, 4),
                60
        );

        planoResponse.setId(idPlano);
        planoResgateResponse.setId(idPlano);
    }

    @Test
    public void aporteExtraTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2024, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        when(planoService.busca(any())).thenReturn(planoResponse);
        when(produtoService.busca(any())).thenReturn(produtoResponse);

        when(planoService.planoCancelado(any())).thenReturn(false);
        when(resgateService.resgateValido(any(), any(), any())).thenReturn(true);
        when(resgateService.resgataAporte(any())).thenReturn(planoResgateResponse);

        ResponseEntity<GenericResponseDTO> response = resgateController.resgate(resgateRequest);
        assertThat(response.getStatusCode().value()).isEqualTo(200);
    }

    @Test
    public void aporteExtraInvalidRedemptionExceptionTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);

        when(produtoService.busca(any())).thenReturn(produtoResponse);
        when(planoService.busca(any())).thenReturn(planoResponse);

        when(planoService.planoCancelado(any())).thenReturn(false);
        when(resgateService.resgateValido(any(), any(), any())).thenReturn(false);

        assertThrows(
                InvalidRedemptionException.class,
                () -> resgateController.resgate(resgateRequest),
                "Expected resgate(resgateRequest) to throw InvalidRedemptionException, but it didn't"
        );
    }

    @Test
    public void aporteExtraCancelledPlanExceptionTest() {
        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produtoResponse.setId(idProduto);
        
        when(planoService.planoCancelado(any())).thenReturn(true);

        assertThrows(
                CancelledPlanException.class,
                () -> resgateController.resgate(resgateRequest),
                "Expected resgate(resgateRequest) to throw CancelledPlanException, but it didn't"
        );
    }

}
